/* Table styling for content types to imitate ckeditor 5 */
.h5p-iframe {
  /* The figure around the table */
  figure.table {
    display: table;
    table-layout: fixed;
    margin: 0 auto;
    padding: 0;
    float: left;

    /* The actual table */
    table {
      border-collapse: collapse;
      height: 100%;
      width: 100%;
      border-spacing: 0;
      border-width: 1px;
      border-color:  #494949;

      td, th {
        padding: 1px;
        border-color:  #494949;
        border-bottom-style: solid;
      }

      td {
        border-width: 0.083em;
      }

      th {
        text-align: left;
        border-width: .167em;
      }

      tr:last-child > td {
        border-bottom-style: none;
      }
    }

    figcaption {
      background-color: transparent;
      caption-side: top;
      color: #333;
      display: table-caption;
      font-size: .75em;
      outline-offset: -1px;
      padding: .6em;
      text-align: center;
      word-break: break-word;
    }
  }

  .table-overflow-protection {
    clear: both;
  }
}
