
// THIS FILE IS AUTO-GENERATED BY STYLE DICTIONARY — DO NOT EDIT DIRECTLY.

$mds-color-lg: rgba(0, 0, 0, 0.2) !default;
$mds-color-md: rgba(0, 0, 0, 0.2) !default;
$mds-color-sm: rgba(0, 0, 0, 0.1) !default;
$mds-blur-lg: 3rem !default;
$mds-blur-md: 1rem !default;
$mds-blur-sm: 0.25rem !default;
$mds-offset-lg: 1rem !default;
$mds-offset-md: 0.5rem !default;
$mds-offset-sm: 0.125rem !default;
