{{!
    This file is part of Moodle - http://moodle.org/

    Moodle is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    Moodle is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with Moodle.  If not, see <http://www.gnu.org/licenses/>.
}}
{{!
    @template core_message/message_drawer_view_contacts_body_section_requests_list

    This template will render a list of contacts for the message drawer.

    Classes required for JS:
    * none

    Data attributes required for JS:
    * All data attributes are required

    Context variables required for this template:
    * userid The logged in user id
    * urls The URLs for the popover

    Example context (json):
    {}

}}

{{#requests}}
    <a
        href="#"
        class="p-2 d-flex list-group-item list-group-item-action border-0 align-items-center"
        data-route="view-conversation"
        {{#conversationid}}
            data-route-param-1="{{.}}"
        {{/conversationid}}
        {{^conversationid}}
            data-route-param-1="false"
            data-route-param-2="create"
            data-route-param-3="{{id}}"
        {{/conversationid}}
        data-request-id="{{id}}"
        data-region="contact-request"
        role="button"
    >
        <img
            class="rounded-circle"
            src="{{{profileimageurl}}}"
            alt="{{fullname}}"
            aria-hidden="true"
            style="height: 38px"
        >
        <div class="w-100 text-truncate ms-2">
            <div class="d-flex">
                <strong class="m-0">
                    {{fullname}}
                </strong>
            </div>
            <p class="m-0 fw-light text-truncate">
                {{#str}} wouldliketocontactyou, core_message {{/str}}
            </p>
        </div>
    </a>
{{/requests}}