<?php
// This file is part of Moodle - http://moodle.org/
//
// Moodle is free software: you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation, either version 3 of the License, or
// (at your option) any later version.
//
// Moodle is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with Moodle.  If not, see <http://www.gnu.org/licenses/>.

/**
 * Strings for component smsgateway_modica, language 'en'.
 *
 * @package    smsgateway_modica
 * @copyright  2025 Safat Shahin <safat.shahin@moodle.com>
 * @license    http://www.gnu.org/copyleft/gpl.html GNU GPL v3 or later
 */

$string['modica_application_name'] = 'Modica application name';
$string['modica_application_password'] = 'Modica application password';
$string['modica_information'] = 'Complete the following fields using the information provided by Modica';
$string['modica_url'] = 'Modica API URL';
$string['pluginname'] = 'Modica';
$string['privacy:metadata'] = 'The Modica SMS gateway plugin does not store any personal data.';
