
// THIS FILE IS AUTO-GENERATED BY STYLE DICTIONARY — DO NOT EDIT DIRECTLY.

@forward "borders";
@forward "colors";
@forward "primitives";
@forward "shadows";
@forward "sizes";
@forward "spacing";
@forward "typography";
