{{!
    This file is part of Moodle - http://moodle.org/

    Moodle is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    Moodle is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with Moodle.  If not, see <http://www.gnu.org/licenses/>.
}}
{{!
    @template tool_componentlibrary/clipboardbutton

    Example context (json):
    {
    }
}}
<div class="bd-clipboard">
    <button type="button" class="btn-clipboard" data-bs-toggle="tooltip" data-bs-placement="top" title="{{#str}}copytoclipboard, tool_componentlibrary{{/str}}"
            data-action="copytoclipboard" data-clipboard-target="{{clipboardtarget}}">
        {{#str}}copy, tool_componentlibrary{{/str}}
    </button>
</div>
