
// THIS FILE IS AUTO-GENERATED BY STYLE DICTIONARY — DO NOT EDIT DIRECTLY.

$mds-icons-lg: 1.25rem !default;
$mds-icons-md: 1rem !default;
$mds-icons-sm: 0.75rem !default;
$mds-icons-xl: 1.5rem !default;
$mds-icons-xs: 0.5rem !default;
$mds-icons-xxl: 2rem !default;
$mds-icons-xxxl: 3rem !default;
