{{!
    This file is part of Moodle - http://moodle.org/

    Moodle is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    Moodle is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with Moodle.  If not, see <http://www.gnu.org/licenses/>.
}}
{{!
    @template core_course/completion_automatic

    Template for displaying an automatic completion rule and its status.

    Example context (json):
    {
        "statuscomplete": 1,
        "description": "View",
        "istrackeduser": true,
        "accessibledescription": "Done: View (Set by Admin User)"
    }
}}
{{#istrackeduser}}
    {{#statuscomplete}}
    <li>
        <div class="automatic-completion-is-complete" {{!
            }}{{#accessibledescription}}{{!
                }}title="{{.}}" {{!
                }}aria-label="{{.}}" {{!
            }}{{/accessibledescription}}>
            {{#pix}}i/checked{{/pix}}
            <strong>{{#str}}completion_automatic:done, core_course{{/str}}</strong>&nbsp;<span class="fw-normal">{{description}}</span>
        </div>
    </li>
    {{/statuscomplete}}
    {{#statuscompletefail}}
    <li>
        <div class="automatic-completion-is-failed" {{!
            }}{{#accessibledescription}}{{!
                }}title="{{.}}" {{!
                }}aria-label="{{.}}" {{!
            }}{{/accessibledescription}}>
            {{#pix}}e/cancel{{/pix}}
            <strong>{{#str}}completion_automatic:failed, core_course{{/str}}</strong>&nbsp;<span class="fw-normal">{{description}}</span>
        </div>
    </li>
    {{/statuscompletefail}}
    {{#statusincomplete}}
    <li>
        <div class="automatic-completion-is-incomplete" {{!
            }}{{#accessibledescription}}{{!
                }}title="{{.}}" {{!
                }}aria-label="{{.}}" {{!
            }}{{/accessibledescription}}>
            <span class="visually-hidden">{{#str}}completion_automatic:todo, core_course{{/str}}</span>
            <span class="fw-normal">{{description}}</span>
        </div>
    </li>
    {{/statusincomplete}}
{{/istrackeduser}}
{{^istrackeduser}}
    <li class="fw-normal">{{description}}</li>
{{/istrackeduser}}
