<?php
// This file is part of Moodle - http://moodle.org/
//
// Moodle is free software: you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation, either version 3 of the License, or
// (at your option) any later version.
//
// Moodle is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with Moodle.  If not, see <http://www.gnu.org/licenses/>.

/**
 * Strings for component 'imscp', language 'en', branch 'MOODLE_20_STABLE'
 *
 * @package mod_imscp
 * @copyright  1999 onwards Martin Dougiamas  {@link http://moodle.com}
 * @license    http://www.gnu.org/copyleft/gpl.html GNU GPL v3 or later
 */

$string['areabackup'] = 'Package archive';
$string['areacontent'] = 'Content';
$string['contentheader'] = 'Content';
$string['deploymenterror'] = 'Content package error!';
$string['keepold'] = 'Archive packages';
$string['keepoldexplain'] = 'How many packages should be archived?';
$string['indicator:cognitivedepth'] = 'IMS pkg cognitive';
$string['indicator:cognitivedepth_help'] = 'This indicator is based on the cognitive depth reached by the student in an IMS content package resource.';
$string['indicator:cognitivedepthdef'] = 'IMS pkg cognitive';
$string['indicator:cognitivedepthdef_help'] = 'The participant has reached this percentage of the cognitive engagement offered by the IMS Package resources during this analysis interval (Levels = No view, View)';
$string['indicator:cognitivedepthdef_link'] = 'Learning_analytics_indicators#Cognitive_depth';
$string['indicator:socialbreadth'] = 'IMS pkg social';
$string['indicator:socialbreadth_help'] = 'This indicator is based on the social breadth reached by the student in an IMS content package resource.';
$string['indicator:socialbreadthdef'] = 'IMS pkg social';
$string['indicator:socialbreadthdef_help'] = 'The participant has reached this percentage of the social engagement offered by the IMS Package resources during this analysis interval (Levels = No participation, Participant alone)';
$string['indicator:socialbreadthdef_link'] = 'Learning_analytics_indicators#Social_breadth';
$string['imscp:addinstance'] = 'Add a new IMS content package';
$string['imscp:view'] = 'View IMS content';
$string['modulename'] = 'IMS content package';
$string['modulename_help'] = '###### Key features
* Add IMS content packages as zip files
* Display content across multiple pages with navigation

###### Ways to use it
* Display a multimedia learning module created with IMS authoring software
* Import a vendor-provided interactive lesson';
$string['modulename_link'] = 'mod/imscp/view';
$string['modulename_summary'] = 'Upload and display existing IMS Global Learning Consortium (IMS) content packages in your course. IMS is commonly used with third-party courseware.';
$string['modulenameplural'] = 'IMS content packages';
$string['navigation'] = 'Navigation';
$string['page-mod-imscp-x'] = 'Any IMS content package module page';
$string['packagefile'] = 'Package file';
$string['pluginadministration'] = 'IMSCP administration';
$string['pluginname'] = 'IMS content package';
$string['privacy:metadata'] = 'The IMS content package plugin does not store any personal data.';
$string['search:activity'] = 'IMS content package - resource information';
$string['toc'] = 'TOC';
