
// THIS FILE IS AUTO-GENERATED BY STYLE DICTIONARY — DO NOT EDIT DIRECTLY.

$mds-bg-feedback-danger-default: #ca3120 !default;
$mds-bg-feedback-danger-subtle: #f4d6d2 !default;
$mds-bg-feedback-info-default: #008196 !default;
$mds-bg-feedback-info-subtle: #cce6ea !default;
$mds-bg-feedback-primary-default: #0f6cbf !default;
$mds-bg-feedback-primary-subtle: #cfe2f2 !default;
$mds-bg-feedback-secondary-subtle: #f8f9fa !default;
$mds-bg-feedback-success-default: #357a32 !default;
$mds-bg-feedback-success-subtle: #d7e4d6 !default;
$mds-bg-feedback-warning-default: #f0ad4e !default;
$mds-bg-feedback-warning-subtle: #fcefdc !default;
$mds-bg-interactive-danger-active: #791d13 !default;
$mds-bg-interactive-danger-default: #ca3120 !default;
$mds-bg-interactive-danger-disabled: #df8379 !default;
$mds-bg-interactive-danger-hover: #a2271a !default;
$mds-bg-interactive-primary-active: #094173 !default;
$mds-bg-interactive-primary-default: #0f6cbf !default;
$mds-bg-interactive-primary-disabled: #6fa7d9 !default;
$mds-bg-interactive-primary-hover: #0c5699 !default;
$mds-bg-interactive-secondary-active: #ced4da !default;
$mds-bg-interactive-secondary-default: #ced4da !default;
$mds-bg-interactive-secondary-disabled: #e9ecef !default;
$mds-bg-interactive-secondary-hover: #dee2e6 !default;
$mds-bg-surface-default: #ffffff !default;
$mds-bg-surface-strong: #e9ecef !default;
$mds-bg-surface-subtle: #f8f9fa !default;
$mds-border-default: #dee2e6 !default;
$mds-border-feedback-danger: #eaada6 !default;
$mds-border-feedback-info: #99cdd5 !default;
$mds-border-feedback-primary: #9fc4e5 !default;
$mds-border-feedback-secondary: #e9ecef !default;
$mds-border-feedback-success: #aecaad !default;
$mds-border-feedback-warning: #f9deb8 !default;
$mds-border-interactive-danger-active: #a2271a !default;
$mds-border-interactive-danger-default: #ca3120 !default;
$mds-border-interactive-danger-disabled: #df8379 !default;
$mds-border-interactive-danger-hover: #ca3120 !default;
$mds-border-interactive-primary-active: #0c5699 !default;
$mds-border-interactive-primary-default: #0f6cbf !default;
$mds-border-interactive-primary-disabled: #6fa7d9 !default;
$mds-border-interactive-primary-hover: #0f6cbf !default;
$mds-border-interactive-secondary-active: #495057 !default;
$mds-border-interactive-secondary-default: #6a737b !default;
$mds-border-interactive-secondary-disabled: #8f959e !default;
$mds-border-interactive-secondary-hover: #6a737b !default;
$mds-border-subtle: #e9ecef !default;
$mds-border-translucent: rgba(0, 0, 0, 0) !default;
$mds-focus-default: #0f6cbf !default;
$mds-text-danger: #ca3120 !default;
$mds-text-default: #1d2125 !default;
$mds-text-emphasis: #000000 !default;
$mds-text-feedback-danger: #51140d !default;
$mds-text-feedback-info: #00343c !default;
$mds-text-feedback-primary: #062b4c !default;
$mds-text-feedback-secondary: #343a40 !default;
$mds-text-feedback-success: #153114 !default;
$mds-text-feedback-warning: #60451f !default;
$mds-text-inverse: #f8f9fa !default;
$mds-text-link-primary-default: #0f6cbf !default;
$mds-text-link-primary-hover: #0c5699 !default;
$mds-text-muted: #6a737b !default;
$mds-text-subtle: #495057 !default;
