{{!
    This file is part of Moodle - http://moodle.org/

    Moodle is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    Moodle is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with Moodle.  If not, see <http://www.gnu.org/licenses/>.
}}
{{!
    @template tool_componentlibrary/examples/formelements/toggles

    Example context (json):
    {
    }
}}
<div class='border p-3 m-3'>
    <h3>{{#str}}toggleviewinfo, tool_componentlibrary{{/str}}</h3>
    <a href="{{{ defaulturl }}} ">{{#str}}showdefault, tool_componentlibrary{{/str}}</a> /
    <a href="{{{ helpurl }}}">{{#str}}showhelp, tool_componentlibrary{{/str}}</a> /
    <a href="{{{ requiredurl }}}">{{#str}}showrequired, tool_componentlibrary{{/str}}</a> /
    <a href="{{{ bothurl }}}">{{#str}}showboth, tool_componentlibrary{{/str}}</a> /
    <a href="{{{ mixedurl }}}">{{#str}}showmixed, tool_componentlibrary{{/str}}</a>
</div>
