{{!
    This file is part of Moodle - http://moodle.org/

    Moodle is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    Moodle is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with Moodle.  If not, see <http://www.gnu.org/licenses/>.
}}
{{!
    @template mod_forum/grades/grader/discussion/post_modal

    Template to render a single post from a discussion.

    Classes required for JS:
    * none

    Data attributes required for JS:
    * data-region="posts-modal"

    Example context (json):
    {
    }
}}
<div class="{{#experimentaldisplaymode}}nested-v2-display-mode{{/experimentaldisplaymode}}" data-region="posts-modal">
    {{#posts}}
        {{#experimentaldisplaymode}}{{> mod_forum/forum_discussion_nested_v2_first_post }}{{/experimentaldisplaymode}}
        {{^experimentaldisplaymode}}{{> mod_forum/forum_discussion_nested_post }}{{/experimentaldisplaymode}}
    {{/posts}}
</div>
