{{!
    This file is part of Moodle - http://moodle.org/

    Moodle is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    Moodle is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with Moodle.  If not, see <http://www.gnu.org/licenses/>.
}}
{{!
    @template core_admin/settings_showhide

    Passes the necessary data to show/hide dependant settings.

    Context variables required for this template:
    * dependencies - JSON-serialized data structure describing the dependencies between settings

    Example context (json):
    {
        "dependencies": {
            "s__setting1":{
                "eq":[
                    [
                        "s__setting0"
                    ]
                ]
            },
            "s__setting2":{
                "eq":[
                    [
                        "s__setting1"
                    ]
                ]
            }
        }
    }
}}
<span id="settings-showhide-{{uniqid}}" data-dependencies="{{dependencies}}"></span>

{{#js}}
    require(['core/showhidesettings'], function(ShowHideSettings) {
        ShowHideSettings.init("settings-showhide-{{uniqid}}");
    });
{{/js}}
