{{!
    This file is part of Moodle - http://moodle.org/

    Moodle is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    Moodle is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with Moodle.  If not, see <http://www.gnu.org/licenses/>.
}}
{{!
    @template mod_forum/local/grades/local/grader/status_placeholder

    Classes required for JS:
    * none

    Data attributes required for JS:
    *

    Context variables required for this template:
    *

    Example context (json):
    {}
}}
<div class="bg-pulse-grey mb-1" style="height: 22px; width: 75px"></div>
<div class="bg-pulse-grey ms-auto me-auto" style="height: 19px; width: 60px"></div>
