.h5p-confirmation-dialog-background {
  position: fixed;
  height: 100%;
  width: 100%;
  left: 0;
  top: 0;
  background: rgba(44, 44, 44, 0.9);
  background-color: hsl(from var(--h5p-theme-main-cta-base) h calc(s * 0.3) 18% / 40%);
  opacity: 1;
  visibility: visible;
  -webkit-transition: opacity 0.1s, linear 0s, visibility 0s linear 0s;
  transition: opacity 0.1s linear 0s, visibility 0s linear 0s;
  z-index: 201;
}
.h5p-confirmation-dialog-background.hidden {
  display: none;
}
.h5p-confirmation-dialog-background.hiding {
  opacity: 0;
  visibility: hidden;
  -webkit-transition: opacity 0.1s, linear 0s, visibility 0s linear 0.1s;
  transition: opacity 0.1s linear 0s, visibility 0s linear 0.1s;
}
.h5p-confirmation-dialog-popup:focus {
  outline: none;
}
.h5p-confirmation-dialog-popup {
  position: absolute;
  display: flex;
  flex-direction: column;
  justify-content: center;
  box-sizing: border-box;
  max-width: 35em;
  min-width: 25em;
  top: 2em;
  left: 50%;
  -webkit-transform: translate(-50%, 0%);
  -ms-transform: translate(-50%, 0%);
  transform: translate(-50%, 0%);
  color: #555;
  -webkit-transition: transform 0.1s ease-in;
  transition: transform 0.1s ease-in;
}
.h5p-confirmation-dialog-popup.hidden {
  -webkit-transform: translate(-50%, 50%);
  -ms-transform: translate(-50%, 50%);
  transform: translate(-50%, 50%);
}
.h5p-confirmation-dialog-header {
  padding: 1.5em;
  background: #fff;
  color: #356593;
}
.h5p-confirmation-dialog-header-text {
  font-size: 1.25em;
}
.h5p-confirmation-dialog-body {
  background: #fafbfc;
  border-top: solid 1px #dde0e9;
  padding: 1.25em 1.5em;
}
.h5p-confirmation-dialog-text {
  margin-bottom: 1.5em;
}
.h5p-confirmation-dialog-buttons {
  float: right;
}
button.h5p-confirmation-dialog-exit:visited, button.h5p-confirmation-dialog-exit:link, button.h5p-confirmation-dialog-exit {
  position: absolute;
  background: none;
  border: none;
  font-size: 2.5em;
  top: -0.9em;
  right: -1.15em;
  color: #fff;
  cursor: pointer;
  text-decoration: none;
}
button.h5p-confirmation-dialog-exit:focus, button.h5p-confirmation-dialog-exit:hover {
  color: #E4ECF5;
}
.h5p-confirmation-dialog-exit:before {
  font-family: "H5P";
  content: "\e890";
}
.h5p-core-button.h5p-confirmation-dialog-confirm-button {
  @media screen and (max-width: 576px) {
    padding-left: 1.5rem;

    span {
      display: none;
    }
  }

  padding-left: 0.75em;
  margin-bottom: 0;
}
.h5p-core-button.h5p-confirmation-dialog-confirm-button:before {
  content: "\e601";
  margin-top: -6px;
  display: inline-block;
}
.h5p-confirmation-dialog-popup.offline .h5p-confirmation-dialog-buttons {
  float: none;
  text-align: center;
}
.h5p-confirmation-dialog-popup.offline .count-down {
  font-family: Arial;
  margin-top: 0.15em;
  color: #000;
}
.h5p-confirmation-dialog-popup.offline .h5p-confirmation-dialog-confirm-button:before {
  content: "\e90b";
  font-weight: normal;
  vertical-align: text-bottom;
}
.throbber-wrapper {
  display: none;
  position: absolute;
  height: 100%;
  width: 100%;
  top: 0;
  left: 0;
  z-index: 1;
  background: rgba(44, 44, 44, 0.9);
  background-color: hsl(from var(--h5p-theme-main-cta-base) h calc(s * 0.3) 18% / 40%);
}
.throbber-wrapper.show {
  display: block;
}
.throbber-wrapper .throbber-container {
  position: absolute;
  top: 50%;
  left: 50%;
  transform: translate(-50%, -50%);
}
.throbber-wrapper .sending-requests-throbber {
  position: absolute;
  top: 7em;
  left: 50%;
  transform: translateX(-50%);
}
.throbber-wrapper .sending-requests-throbber:before {
  display: block;
  font-family: 'H5P';
  content: "\e90b";
  color: white;
  font-size: 10em;
  animation: request-throbber 1.5s infinite linear;
}
@keyframes request-throbber {
  from {
    transform: rotate(0);
  }
  to {
    transform: rotate(359deg);
  }
}

@media (prefers-reduced-motion) {
  .h5p-confirmation-dialog-background {
    -webkit-transition: none;
    transition: none;

    .h5p-confirmation-dialog-popup {
      -webkit-transition: none;
      transition: none;
    }

    &.hiding {
      .h5p-confirmation-dialog-popup {
        opacity: 0;
      }
    }
  }
}

/* H5P Theme */

.h5p-theme .h5p-confirmation-dialog-popup {
  max-width: 45rem;
  width: calc(80% - 2*var(--h5p-theme-spacing-l));
  min-width: unset;
  color: var(--h5p-theme-text-primary);
}

.h5p-theme .h5p-confirmation-dialog-header {
  display: flex;
  align-items: center;
  color: var(--h5p-theme-text-primary);
  padding: var(--h5p-theme-spacing-m) var(--h5p-theme-spacing-l);
  font-weight: bold;
  background-color: var(--h5p-theme-ui-base);
  border-radius: var(--h5p-theme-border-radius-large) var(--h5p-theme-border-radius-large) 0 0;
}

.h5p-theme .h5p-confirmation-dialog-header-text {
  font-size: var(--h5p-theme-font-size-xl);
  margin-right: 3em;
}

.h5p-theme .h5p-confirmation-dialog-body {
  background: var(--h5p-theme-ui-base);
  border-top: 0px;
  padding: 0 var(--h5p-theme-spacing-l) var(--h5p-theme-spacing-l) var(--h5p-theme-spacing-l);
  border-radius: 0 0 var(--h5p-theme-border-radius-large) var(--h5p-theme-border-radius-large);
}

.h5p-theme button.h5p-confirmation-dialog-exit:visited,
.h5p-theme button.h5p-confirmation-dialog-exit:link,
.h5p-theme button.h5p-confirmation-dialog-exit {
  font-size: var(--h5p-theme-font-size-xl);
  color: var(--h5p-theme-text-primary);
  top: var(--h5p-theme-spacing-s);
  right: var(--h5p-theme-spacing-s);
  line-height: calc(var(--h5p-theme-spacing-s) * 3.0836);
  padding: 0;
}

.h5p-theme button.h5p-confirmation-dialog-exit:focus-visible {
  outline: 2px ridge var(--h5p-theme-contrast-cta-white);
  border-radius: 100%;
  outline-offset: 2px;
}

.h5p-theme .h5p-confirmation-dialog-exit:before {
  transform: translateY(-0.48em);
  font-family: "h5p-theme";
  content: "\e910";
  padding: var(--h5p-theme-spacing-s);
  border-radius: 50%;
}

.h5p-theme .h5p-confirmation-dialog-exit:hover:before {
  color: var(--h5p-theme-text-primary);
  background-color: var(--h5p-theme-alternative-base);
}

/* TODO: check if code above makes this redundant */
.h5p-theme .h5p-core-button:focus-visible {
  background-color: var(--h5p-theme-contrast-cta-white);
}

.h5p-theme .h5p-confirmation-dialog-background {
  background-color: rgba(0,0,0,0.85); /* Fallback color */
  background-color: hsl(from var(--h5p-theme-main-cta-base) h calc(s * 0.3) 18% / 40%);
}

.h5p-theme .h5p-confirmation-dialog-buttons {
  container-type: inline-size;
  container-name: h5p-confirmation-dialog-buttons;
  display: flex;
  flex-wrap: wrap;
  justify-content: flex-end;
  gap: var(--h5p-theme-spacing-s);
  width: 100%;
}

.h5p-theme .h5p-confirmation-dialog-buttons .h5p-theme-secondary-cta {
  padding-inline: calc(var(--h5p-theme-spacing-l));
}

.h5p-theme .h5p-core-button.h5p-confirmation-dialog-confirm-button:before {
  content: "\e908";
  margin-top: unset;
  font-size: unset;
  font-family: "h5p-theme";
  display: inline-block;
  line-height: unset;
  padding-right: unset;
}

@container (max-width: 400px) {
  .h5p-theme .h5p-confirmation-dialog-buttons button.h5p-theme-cancel::before {
    content: "\e910";
    font-family: "h5p-theme";
    font-size: 90%;
  }

  .h5p-theme .h5p-confirmation-dialog-buttons .h5p-theme-primary-cta:hover {
    padding-inline: var(--h5p-theme-spacing-l);
  }

  .h5p-theme .h5p-confirmation-dialog-buttons .h5p-theme-primary-cta:disabled:before,
  .h5p-theme .h5p-confirmation-dialog-buttons .h5p-theme-secondary-cta:disabled:before,
  .h5p-theme .h5p-confirmation-dialog-buttons .h5p-theme-primary-cta:before {
    margin-right: 0;
    opacity: 1;
    padding: 0;
    position: unset;
    text-indent: 0;
    transform: translate(0, 0);
  }

  .h5p-theme .h5p-confirmation-dialog-buttons .h5p-theme-secondary-cta:before {
    margin-right: 0;
  }

  .h5p-theme .h5p-confirmation-dialog-buttons span {
    display: none;
  }
}

@container (max-width: 250px) {
  .h5p-theme .h5p-confirmation-dialog-buttons .h5p-theme-primary-cta:hover {
    padding-inline: var(--h5p-theme-spacing-s);
  }
  .h5p-theme .h5p-confirmation-dialog-buttons .h5p-theme-secondary-cta {
    padding-inline: var(--h5p-theme-spacing-s);
  }
}
